#include "COPY.c"
#include "README.c"
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "hash.c"
#include "tprep.c"
#include "input.c"
#include "font.c"
#include "renderall.c"

char *get_arg_str(int *argc, char **argv)
{
  char *R;
  int i;
  if (*argc<=2)
     exit(fprintf(stderr,"%s: option '%s' needs an argument\n",
                  argv[0],argv[1]));
  R= argv[2];
  for(i=1;i<*argc-2;i++)
    argv[i]= argv[i+2]; 
  (*argc)-= 2;
  return R;
}

void shift_args(int *argc, char **argv)
{
  int i;
  for(i=1;i<*argc-1;i++)
    argv[i]= argv[i+1]; 
  (*argc)-= 1;
}

int get_arg_int(int *argc, char **argv)
{
  return atoi(get_arg_str(argc,argv));
}

char *font_file, *hash_name, *func_name;
int font_size;
int opt_dpi= 96;
int opt_genhash, opt_utflib, render_scope;


void read_args(int argc,char **argv)
{
  while(argc>1)
  {
    if (!strcmp(argv[1],"-f"))       { font_file= get_arg_str(&argc, argv); }
    else if (!strcmp(argv[1], "-s")) { font_size= get_arg_int(&argc,argv); }
    else if (!strcmp(argv[1], "-h")) { hash_name= get_arg_str(&argc,argv); }
    else if (!strcmp(argv[1], "-r")) { func_name= get_arg_str(&argc,argv);
                                       render_scope= 0; }
    else if (!strcmp(argv[1], "-R")) { func_name= get_arg_str(&argc,argv);
                                       render_scope= 2; }
    else if (!strcmp(argv[1], "-u")) { opt_utflib= 1; shift_args(&argc,argv); }
    else if (!strcmp(argv[1], "-U")) { opt_utflib= 2; shift_args(&argc,argv); }
    else if (!strcmp(argv[1], "-g")) { opt_genhash=1;shift_args(&argc,argv); }
    else if (!strcmp(argv[1], "-G")) { opt_genhash=2;shift_args(&argc,argv); }
    else if (!strcmp(argv[1], "-d")) { opt_dpi= get_arg_int(&argc,argv); }
    else goto usage_error;
  }
  if (!font_file || !font_size || !hash_name || !func_name)
     goto usage_error;
  return ;

usage_error:
   fputs(
#include "README.xc"
   ,stderr);
   exit(1);
#if 0
   exit(fprintf(stderr,
      "usage: %s -f font_file -s size -h hash_func_name -r render_func_name\n"
      "optional arguments:\n"
      "   -u    generates the utf8 decoding function also.\n"
      "   -U    same as -u but the function is declared as static.\n"
      "   -g    generates a hash function with the given name for the set.\n"
      "   -G    same as -g, but static function.\n"
      "   -d dpi\n"
      "         Default for this value is 96.\n"
      "You can also use -R instead of -r, which again makes the rendering\n"
      "function static.\n"
        ,argv[0]));
#endif
}

static const char *render_skel_code= 
#include "render-skel.xc"
;

static const char *utf_lib_code= 
#include "libutf8.xc"
;

int main(int argc,char **argv)
{
  unsigned int *chars; int nchars;
  char *code,*P;
  int fontA,fontH;

  glyph_t *G;
  read_args(argc,argv);
  read_input(&chars, &nchars);

  if (font_load(font_file, font_size, opt_dpi))
    exit(1);

  G= render_glyphs(chars, nchars,&fontA, &fontH);

  if (opt_utflib)
  {
    if (opt_utflib==2) printf("static ");
    puts(utf_lib_code);
  }

  if (opt_genhash)
  {
    if (opt_genhash==2) printf("static ");
    printf("unsigned int %s(unsigned int V)\n",hash_name);
#if 0
       printf("{ return 0;\n}\n");  // XX
#else
    print_hash_func(chars, nchars);
#endif
  }

  if (render_scope==2) printf("static ");

  code= tp_replace_str(render_skel_code, "RENDER", func_name, 0);
  code= tp_replace_str(code, "HASH", hash_name, 1);
  code= tp_replace_int(code, "SENTINEL", nchars,1);
  code= tp_replace_int(code, "FONT_HEIGHT", fontH,1);
  code= tp_replace_int(code, "FONT_ASCENT", fontA,1);

  while(1)
  {
    char *N,*K;
    P= strstr(code, "\n@");
    if (!P) break;
    P++;
    fwrite(code, 1, P-code, stdout);
    K= strchr(P,'\n'); N= K+1;
    K[0]= 0; while(isspace(K[-1])) { K[-1]=0; K--; } 
    if (!strcmp(P,"@FONTDATA")) print_font_data(G, nchars);
    if (!strcmp(P,"@GLMAP")) print_font_map(G, nchars);
    code= N;
  }
  puts(code);

  return 0;
}
